package edu.travel.datasource;

import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.inner.InnerInterceptor;
import net.sf.jsqlparser.JSQLParserException;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;
import net.sf.jsqlparser.expression.StringValue;
import net.sf.jsqlparser.expression.operators.conditional.AndExpression;
import net.sf.jsqlparser.expression.operators.relational.EqualsTo;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SelectBody;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Field;
import java.sql.SQLException;

/**
 * @anthor 畅
 *  处理基础架构中统一带上 project 字段
 */
public class ProjectInterceptor implements InnerInterceptor {
    Logger logger = LoggerFactory.getLogger(ProjectInterceptor.class);
    private String convertDeleteToUpdate(String originalSql) {
        // 示例：将DELETE FROM table WHERE ... 转为 UPDATE table SET deleted=1 WHERE ...
        String newUPDATE = originalSql.replaceFirst("DELETE FROM", "UPDATE");
        if (newUPDATE.contains("where")||newUPDATE.contains("WHERE")) {
           newUPDATE =  newUPDATE.replace("WHERE", "SET delete_flag=1 WHERE ");
        }else {
            newUPDATE =  "";
        }
        return newUPDATE;

    }

    @Override
    public void beforeUpdate(Executor executor, MappedStatement ms, Object parameter) throws SQLException {
        BoundSql boundSql = ms.getBoundSql(parameter);
        if (ms.getSqlCommandType().equals(SqlCommandType.DELETE)) {
            // 转换DELETE为UPDATE，此处需复杂处理，示例简化
            // 实际需解析原始SQL并替换，此处仅为思路
            String newSql = convertDeleteToUpdate(boundSql.getSql());
            // 反射修改BoundSql中的SQL
            Class<? extends BoundSql> aClass = boundSql.getClass();
            Field declaredField = null;
            try {
                declaredField = aClass.getDeclaredField("sql");
                declaredField.setAccessible(true);
                declaredField.set(boundSql,newSql);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }

        }
    }

    @Override
    public void beforeQuery(Executor executor, MappedStatement ms, Object parameter, RowBounds rowBounds, ResultHandler resultHandler, BoundSql boundSql) throws SQLException {
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String project = request.getHeader("project");
        if (StringUtils.isNotBlank(project)) {
            String originSql = boundSql.getSql();
            if (originSql.contains("join") || originSql.contains("JOIN")) {
                return;
            }
            logger.debug("origin sql: " + originSql);
            //获取 Statement 执行sql的对象
            Statement statement = null;
            try {
                statement = CCJSqlParserUtil.parse(originSql);
            } catch (JSQLParserException e) {
                throw new RuntimeException(e);
            }
            Select select = (Select) statement;

            SelectBody selectBody = select.getSelectBody();
            if (selectBody == null) {
                return;
            }
            String targetSql = "";
            if (selectBody instanceof PlainSelect) {
                PlainSelect plainSelect = (PlainSelect) selectBody;
                Expression where = plainSelect.getWhere();
                EqualsTo projectEq = new EqualsTo();
                projectEq.setLeftExpression(new Column("project"));
                projectEq.setRightExpression(new StringValue(project));
                EqualsTo deleteFlag = new EqualsTo();
                deleteFlag.setLeftExpression(new Column("delete_flag"));
                deleteFlag.setRightExpression(new LongValue(0));
                String plainSelectString = plainSelect.toString();
                if (!plainSelectString.contains("project = ") && !plainSelectString.contains("delete_flag = ")) {
                    AndExpression targetExpression = new AndExpression(projectEq, deleteFlag);
                    if (plainSelectString.contains("where") || plainSelectString.contains("WHERE")) {
                        AndExpression andExpression = new AndExpression(where, targetExpression);
                        plainSelect.setWhere(andExpression);
                    }else {
                        plainSelect.setWhere(targetExpression);
                    }

                    targetSql = plainSelect.toString();
                }else {
                    targetSql = plainSelect.toString();
                }

            }
            // 修改完成的sql 再设置回去
            PluginUtils.MPBoundSql mpBoundSql = PluginUtils.mpBoundSql(boundSql);
            System.out.println(targetSql);
            mpBoundSql.sql(targetSql);

        }
    }
}
