package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.constant.BaseConstant;
import edu.travel.commodity.dto.ProductSpecDto;
import edu.travel.commodity.entity.*;
import edu.travel.commodity.mapper.ShopProductSpecMapper;
import edu.travel.commodity.service.*;
import edu.travel.commodity.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ShopProductSpecServiceImpl extends ServiceImpl<ShopProductSpecMapper, ShopProductSpec> implements ShopProductSpecService {
    @Autowired
    private ShopProductService shopProductService;
    @Autowired
    private ShopSpecValueService shopSpecValueService;
    @Autowired
    private ShopProductSkuService shopProductSkuService;
    @Autowired
    private ShopProductImageService shopProductImageService;
    @Autowired
    private ShopProductSpecService shopProductSpecService;
    @Autowired
    private ShopProductSpecMapper shopProductSpecMapper;

    @Override
    public ProductSpecVo getShopSpec(ProductSpecDto param) {
        LambdaQueryWrapper<ShopProduct> query =  Wrappers.<ShopProduct>lambdaQuery().eq(ShopProduct::getId, param.getProductId())
                .eq(ShopProduct::getCountryId, param.getCountryId());
        //商品信息
        ShopProduct one = shopProductService.getOne(query);
        if(ObjectUtil.isEmpty(one)){
            return null;
        }

        List<ShopProductSku> list1 = shopProductSkuService.list(Wrappers.<ShopProductSku>lambdaQuery()
                .eq(ShopProductSku::getStatus, BaseConstant.BASIC_STATUS_NO_NUM)
                .eq(ShopProductSku::getProductId, one.getId()));

        if(ObjectUtil.isEmpty(list1)){
            return null;
        }

        ProductSpecVo bean = BeanUtil.toBean(one, ProductSpecVo.class);
        //商品的全部图片
        List<ShopProductImage> list = shopProductImageService.lambdaQuery()
                .eq(ShopProductImage::getProductId, one.getId()).list();
        List<ProductImageVo> productImageVos = BeanUtil.copyToList(list, ProductImageVo.class);

        List<ProductImageVo> collect = productImageVos.stream().filter(item -> item.getIsDefault().equals(BaseConstant.BASIC_IS_DEFAULT_YES)).collect(Collectors.toList());
        bean.setImages(collect);

        Map<String, List<ProductImageVo>> imageMap = collect.stream().collect(Collectors.groupingBy(ProductImageVo::getSkuId));
        //SKU
        Set<String> skus = list1.stream().map(item->item.getSkuId().toString()).collect(Collectors.toSet());

        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String project = request.getHeader("project");

        //sku规格和规格对应的值
        List<ShopSkuSpecValueVo> skuValue=shopProductSpecMapper.getSkuValue(skus,project);
        Map<String, List<ShopSkuSpecValueVo>> map = skuValue.stream().collect(Collectors.groupingBy(ShopSkuSpecValueVo::getSkuId));

        ArrayList<ProductSkuVo> productSkuVos = new ArrayList<>();
        for (ShopProductSku info : list1) {
            ProductSkuVo infoVo = BeanUtil.toBean(info, ProductSkuVo.class);
            infoVo.setDescription(one.getDescription());
            infoVo.setProductName(one.getProductName());
            bean.setImages(imageMap.get(info.getSkuId().toString()));
            List<ShopSkuSpecValueVo> shopSkuSpecValueVos = map.get(info.getSkuId().toString());
            infoVo.setSpecValue(shopSkuSpecValueVos);
            productSkuVos.add(infoVo);
        }
        List<SpecSortVo> specSortVoList= getSpecSortVoList(param.getProductId());
        bean.setSpecSortVoList(specSortVoList);
        bean.setSkuList(productSkuVos);
        return bean;
    }


    public  List<SpecSortVo> getSpecSortVoList(String productId){
        List<ShopProductSpec> specs = shopProductSpecService.lambdaQuery().eq(ShopProductSpec::getProductId, productId).orderByAsc(ShopProductSpec::getSortOrder).list();
        List<SpecSortVo> list = BeanUtil.copyToList(specs, SpecSortVo.class);
        if(ObjectUtil.isEmpty(list)){
            return list;
        }
        Set<Long> collect = specs.stream().map(ShopProductSpec::getId).collect(Collectors.toSet());
        List<ShopSpecValue> list1 = shopSpecValueService.lambdaQuery().in(ShopSpecValue::getProductSpecId, collect).list();
        for (SpecSortVo spec : list) {
            List<ShopSpecValue> collect1 = list1.stream().filter(item -> item.getProductSpecId().toString().equals(spec.getId()))
                    .sorted(Comparator.comparing(ShopSpecValue::getSortOrder)).collect(Collectors.toList());
            List<SpecValueSortVo> specValueSortVos = BeanUtil.copyToList(collect1, SpecValueSortVo.class);
            spec.setList(specValueSortVos);
        }
        return list;
    }



}
