package edu.travel.country.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import edu.travel.country.entity.ShopLanguage;
import edu.travel.country.service.ShopLanguageService;
import edu.travel.dto.ShopLanguageDto;
import edu.travel.remote.ShopLanguageRemoteController;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.ShopLanguageVo;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
* 语言表(shop_language)表控制层
*
* @author xxxxx
*/
@RestController
@RequestMapping("/shop_language")
public class ShopLanguageController extends BaseController<ShopLanguage> implements ShopLanguageRemoteController {
    /**
     * 服务对象
     */
@Autowired
    private ShopLanguageService shopLanguageService;

/**
 * 语言分页
 */
@GetMapping("/getLanguagePage")
    public RPCBaseResponse<IPage<ShopLanguageVo>> getLanguagePage(ShopLanguageDto shopLanguageDto) {
        return shopLanguageService.getLanguagePage(shopLanguageDto);
    }

    /**
     * 获取语言详情
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getLanguageFormId")
    public RPCBaseResponse<ShopLanguageVo> getFormId(String id) {
        RPCBaseResponse<ShopLanguage> languageRPCBaseResponse = super.getId(id);
        RPCBaseResponse<ShopLanguageVo> shopLanguageVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(languageRPCBaseResponse, shopLanguageVoRPCBaseResponse);
        return shopLanguageVoRPCBaseResponse;
    }

    /**
     * 更新语言信息
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/updateLanguageFormId")
    public RPCBaseResponse<ShopLanguageVo> updateTargetFormId(@RequestBody ShopLanguageDto entity) {
        ShopLanguage shopLanguage = new ShopLanguage();
        BeanUtils.copyProperties(entity, shopLanguage);
        RPCBaseResponse<ShopLanguage> languageRPCBaseResponse = super.updateTargetById(shopLanguage);
        RPCBaseResponse<ShopLanguageVo> shopLanguageVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(languageRPCBaseResponse, shopLanguageVoRPCBaseResponse);
        return shopLanguageVoRPCBaseResponse;
    }

    /**
     * 新增语言信息
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/saveLanguageForm")
    public RPCBaseResponse<ShopLanguageVo> saveFormTarget(@RequestBody ShopLanguageDto entity) {
        ShopLanguage shopLanguage = new ShopLanguage();
        BeanUtils.copyProperties(entity, shopLanguage);
        RPCBaseResponse<ShopLanguage> languageRPCBaseResponse = super.saveTarget(shopLanguage);
        RPCBaseResponse<ShopLanguageVo> shopLanguageVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(languageRPCBaseResponse, shopLanguageVoRPCBaseResponse);
        return shopLanguageVoRPCBaseResponse;
    }

    /**
     * 删除语言
     * @param ids
     * @return
     */
    @Override
    @PostMapping("/deleteLanguageFormId")
    public RPCBaseResponse<ShopLanguageVo> deleteTargetFormId(@RequestBody List<String> ids) {
        RPCBaseResponse<ShopLanguage> languageRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopLanguageVo> shopLanguageVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(languageRPCBaseResponse, shopLanguageVoRPCBaseResponse);
    return shopLanguageVoRPCBaseResponse;
    }

    /**
     * 获取语言表
     * @return
     */
    @Override
    @GetMapping("/LanguagelistForm")
    public RPCBaseResponse<List<ShopLanguageVo>> getAllForm() {
        RPCBaseResponse<List<ShopLanguage>> listRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopLanguageVo>> shopLanguageVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(listRPCBaseResponse, shopLanguageVoRPCBaseResponse);
        return shopLanguageVoRPCBaseResponse;
    }
}
